# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_BOOM_WARNING_IDX(Base):
    """
    宏观经济景气预警指数
    """

    __tablename__ = "MAC_BOOM_WARNING_IDX"

    id = Column(Integer, primary_key=True)
    stat_month = Column(String(20), nullable=False)
    warning_idx = Column(DECIMAL(10, 4))
    industry_idx_sgn = Column(DECIMAL(10, 4))
    fixed_assets_sgn = Column(DECIMAL(10, 4))
    rpi_sgn = Column(DECIMAL(10, 4))
    import_export_sgn = Column(DECIMAL(10, 4))
    gov_revenue_sgn = Column(DECIMAL(10, 4))
    industry_profit_sgn = Column(DECIMAL(10, 4))
    resident_dpi_sgn = Column(DECIMAL(10, 4))
    loan_sgn = Column(DECIMAL(10, 4))
    m2_sgn = Column(DECIMAL(10, 4))
    cpi_sgn = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_LEND_RATE(Base):
    """
    银行间拆借利率表
    """

    __tablename__ = "MAC_LEND_RATE"

    id = Column(Integer, primary_key=True)
    day = Column(String(20), nullable=False)
    currency_id = Column(Integer, nullable=False)
    currency_name = Column(String(20))
    market_id = Column(Integer, nullable=False)
    term_id = Column(Integer, nullable=False)
    interest_rate = Column(DECIMAL(10, 6), nullable=False)
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_INDUSTRY_INDICATOR(Base):
    """
    工业企业主要指标
    """

    __tablename__ = "MAC_INDUSTRY_INDICATOR"

    id = Column(Integer, primary_key=True)
    stat_month = Column(String(20), nullable=False)
    enterprise_value_acc = Column(Integer)
    loss_enterprise_value_acc = Column(Integer)
    loss_enterprise_ratio_acc = Column(DECIMAL(10, 4))
    current_asset_value_acc = Column(DECIMAL(20, 4))
    current_asset_ratio_acc = Column(DECIMAL(10, 4))
    accounts_receivable_value_acc = Column(DECIMAL(20, 4))
    accounts_receivable_ratio_acc = Column(DECIMAL(10, 4))
    inventories_value_acc = Column(DECIMAL(20, 4))
    inventories_ratio_acc = Column(DECIMAL(10, 4))
    finished_product_value_acc = Column(DECIMAL(20, 4))
    finished_product_ratio_acc = Column(DECIMAL(10, 4))
    total_assets_value_acc = Column(DECIMAL(20, 4))
    total_assets_ratio_acc = Column(DECIMAL(10, 4))
    liabilities_value_acc = Column(DECIMAL(20, 4))
    liabilities_ratio_acc = Column(DECIMAL(10, 4))
    main_business_value_acc = Column(DECIMAL(20, 4))
    main_business_ratio_acc = Column(DECIMAL(10, 4))
    main_business_tax_value_acc = Column(DECIMAL(20, 4))
    main_business_tax_ratio_acc = Column(DECIMAL(10, 4))
    sale_expense_value_acc = Column(DECIMAL(20, 4))
    sale_expense_ratio_acc = Column(DECIMAL(10, 4))
    management_cost_value_acc = Column(DECIMAL(20, 4))
    management_cost_ratio_acc = Column(DECIMAL(10, 4))
    financial_expense_value_acc = Column(DECIMAL(20, 4))
    financial_expense_ratio_acc = Column(DECIMAL(10, 4))
    interest_expense_value_acc = Column(DECIMAL(20, 4))
    interest_expense_ratio_acc = Column(DECIMAL(10, 4))
    total_interest_value_acc = Column(DECIMAL(20, 4))
    total_interest_ratio_acc = Column(DECIMAL(10, 4))
    enterprise_total_loss_value_acc = Column(DECIMAL(20, 4))
    enterprise_total_loss_ratio_acc = Column(DECIMAL(10, 4))
    vat_value_acc = Column(DECIMAL(20, 4))
    vat_ratio_acc = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_AREA_RETAIL_SALE(Base):
    """
    分地区消费品零售总额
    """

    __tablename__ = "MAC_AREA_RETAIL_SALE"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    area_code = Column(String(20), nullable=False)
    area_name = Column(String(100), nullable=False)
    retail = Column(DECIMAL(20, 4))
    retail_yoy = Column(DECIMAL(10, 4))
    city_retail = Column(DECIMAL(20, 4))
    city_county_retail = Column(DECIMAL(20, 4))
    city_county_below_retail = Column(DECIMAL(20, 4))
    city_whole_sale_retail = Column(DECIMAL(20, 4))
    hotels_retail = Column(DECIMAL(20, 4))
    manufacturing_retail = Column(DECIMAL(20, 4))
    agricultural_retail = Column(DECIMAL(20, 4))
    others_retail = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_SALE_RETAIL_MONTH(Base):
    """
    社会消费品销售总额
    """
    __tablename__ = "MAC_SALE_RETAIL_MONTH"

    id = Column(Integer, primary_key=True)
    stat_month = Column(String(20), nullable=False)
    retail_sin = Column(DECIMAL(20, 4))
    retail_acc = Column(DECIMAL(20, 4))
    retail_sin_yoy = Column(DECIMAL(10, 4))
    retail_acc_yoy = Column(DECIMAL(10, 4))
    scale_retail_sin = Column(DECIMAL(20, 4))
    scale_retail_acc = Column(DECIMAL(20, 4))
    scale_retail_sin_yoy = Column(DECIMAL(10, 4))
    scale_retail_acc_yoy = Column(DECIMAL(10, 4))
    city_retail_sin = Column(DECIMAL(20, 4))
    city_retail_acc = Column(DECIMAL(20, 4))
    city_retail_sin_yoy = Column(DECIMAL(10, 4))
    city_retail_acc_yoy = Column(DECIMAL(10, 4))
    rural_retail_sin = Column(DECIMAL(20, 4))
    rural_retail_acc = Column(DECIMAL(20, 4))
    rural_retail_sin_yoy = Column(DECIMAL(10, 4))
    rural_retail_acc_yoy = Column(DECIMAL(10, 4))
    hotel_retail_sin = Column(DECIMAL(20, 4))
    hotel_retail_acc = Column(DECIMAL(20, 4))
    hotel_retail_sin_yoy = Column(DECIMAL(10, 4))
    hotel_retail_acc_yoy = Column(DECIMAL(10, 4))
    hotel_scale_retail_sin = Column(DECIMAL(20, 4))
    hotel_scale_retail_acc = Column(DECIMAL(20, 4))
    hotel_scale_retail_sin_yoy = Column(DECIMAL(10, 4))
    hotel_scale_retail_acc_yoy = Column(DECIMAL(10, 4))
    sale_retail_sin = Column(DECIMAL(20, 4))
    sale_retail_acc = Column(DECIMAL(20, 4))
    sale_retail_sin_yoy = Column(DECIMAL(10, 4))
    sale_retail_acc_yoy = Column(DECIMAL(10, 4))
    sale_scale_retail_sin = Column(DECIMAL(20, 4))
    sale_scale_retail_acc = Column(DECIMAL(20, 4))
    sale_scale_retail_sin_yoy = Column(DECIMAL(10, 4))
    sale_scale_retail_acc_yoy = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)



# coding=utf-8
import sys
import os
sys.modules["ROOT_DIR"] = os.path.abspath(os.path.dirname(__file__))

from .api import *
from .finance_service import *
from .macro import macro
from . import alpha101
from . import alpha191
from . import technical_analysis
from .finance import finance


def auth(username, password, host="39.107.190.114", port=7000):
    from .client import JQDataClient
    JQDataClient.set_auth_params(host=host, port=port, username=username, password=password)


__all__ = [
    "auth",
    "get_price",
    "get_trade_days",
    "get_all_trade_days",
    "get_extras",
    "get_index_stocks",
    "get_industry_stocks",
    "get_industries",
    "get_concept_stocks",
    "get_concepts",
    "get_all_securities",
    "get_security_info",
    "get_money_flow",
    "get_fundamentals",
    "get_fundamentals_continuously",
    "get_billboard_list",
    "get_locked_shares",
    "get_mtss",
    "get_margincash_stocks",
    "get_marginsec_stocks",
    "get_future_contracts",
    "get_dominant_future",
    "normalize_code",
    "query",
    "balance",
    "income",
    "cash_flow",
    "indicator",
    "bank_indicator",
    "security_indicator",
    "insurance_indicator",
    "valuation",
    "macro",
    "alpha101",
    "alpha191",
    "technical_analysis",
    "get_baidu_factor",
    "get_ticks",
    "finance",
    "get_factor_values",
    "get_index_weights",
]

# __all__.extend(fin.__all__)






# coding=utf-8
from functools import wraps
from .utils import *
from .client import JQDataClient


@assert_auth
def get_price(security, start_date=None, end_date=None, frequency='daily', 
    fields=None, skip_paused=False, fq='pre', count=None):
    """
    获取一支或者多只证券的行情数据

    :param security 一支证券代码或者一个证券代码的list
    :param count 与 start_date 二选一，不可同时使用.数量, 返回的结果集的行数, 即表示获取 end_date 之前几个 frequency 的数据
    :param start_date 与 count 二选一，不可同时使用. 字符串或者 datetime.datetime/datetime.date 对象, 开始时间
    :param end_date 格式同上, 结束时间, 默认是'2015-12-31', 包含此日期.
    :param frequency 单位时间长度, 几天或者几分钟, 现在支持'Xd','Xm', 'daily'(等同于'1d'), 'minute'(等同于'1m'), X是一个正整数, 分别表示X天和X分钟
    :param fields 字符串list, 默认是None(表示['open', 'close', 'high', 'low', 'volume', 'money']这几个标准字段), 支持以下属性 ['open', 'close', 'low', 'high', 'volume', 'money', 'factor', 'high_limit', 'low_limit', 'avg', 'pre_close', 'paused']
    :param skip_paused 是否跳过不交易日期(包括停牌, 未上市或者退市后的日期). 如果不跳过, 停牌时会使用停牌前的数据填充, 上市前或者退市后数据都为 nan
    :return 如果是一支证券, 则返回pandas.DataFrame对象, 行索引是datetime.datetime对象, 列索引是行情字段名字; 如果是多支证券, 则返回pandas.Panel对象, 里面是很多pandas.DataFrame对象, 索引是行情字段(open/close/…), 每个pandas.DataFrame的行索引是datetime.datetime对象, 列索引是证券代号.
    """
    security = convert_security(security)
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    if (not count) and (not start_date):
            start_date = "2015-01-01"
    if count and start_date:
        raise ParamsError("(start_date, count) only one param is required")
    return JQDataClient.instance().get_price(**locals())


@assert_auth
def get_extras(info, security_list, start_date=None, end_date=None, df=True, count=None):
    """
    得到多只标的在一段时间的如下额外的数据

    :param info ['is_st', 'acc_net_value', 'unit_net_value', 'futures_sett_price', 'futures_positions'] 中的一个
    :param security_list 证券列表
    :param start_date 开始日期
    :param end_date 结束日期
    :param df 返回pandas.DataFrame对象还是一个dict
    :param count 数量, 与 start_date 二选一, 不可同时使用, 必须大于 0
    :return <df=True>:pandas.DataFrame对象, 列索引是股票代号, 行索引是datetime.datetime；<df=False>:一个dict, key是基金代号, value是numpy.ndarray
    """
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    security_list = convert_security(security_list)
    return JQDataClient.instance().get_extras(**locals())


@assert_auth
def get_fundamentals(query_object, date=None, statDate=None):
    """
    查询财务数据, 详细的数据字段描述在 https://www.joinquant.com/data/dict/fundamentals 中查看

    :param query_object 一个sqlalchemy.orm.query.Query对象
    :param date 查询日期, 一个字符串(格式类似’2015-10-15’)或者datetime.date/datetime.datetime对象, 可以是None, 使用默认日期
    :param statDate: 财报统计的季度或者年份, 一个字符串, 有两种格式:1.季度: 格式是: 年 + ‘q’ + 季度序号, 例如: ‘2015q1’, ‘2013q4’. 2.年份: 格式就是年份的数字, 例如: ‘2015’, ‘2016’.
    :return 返回一个 pandas.DataFrame, 每一行对应数据库返回的每一行(可能是几个表的联合查询结果的一行), 列索引是你查询的所有字段;为了防止返回数据量过大, 我们每次最多返回10000行;当相关股票上市前、退市后，财务数据返回各字段为空
    """
    from .finance_service import get_fundamentals_sql
    if date is None and statDate is None:
        date = datetime.date.today()
        from .calendar_service import CalendarService
        trade_days = CalendarService.get_all_trade_days()
        date = list(filter(lambda item: item < date, trade_days))[-1]
    elif date:
        yesterday = datetime.date.today() - datetime.timedelta(days=1)
        date = min(to_date(date), yesterday)
    sql = get_fundamentals_sql(query_object, date, statDate)
    return JQDataClient.instance().get_fundamentals(sql=sql)


@assert_auth
def get_fundamentals_continuously(query_object, end_date=None, count=None):
    """
    查询财务数据，详细的数据字段描述在 https://www.joinquant.com/data/dict/fundamentals 中查看
    :param query_object:一个sqlalchemy.orm.query.Query对象
    :param end_date:查询日期, 一个字符串(格式类似’2015-10-15’)或者datetime.date/datetime.datetime对象, 可以是None, 使用默认日期
    :param count:获取 end_date 前 count 个日期的数据
    :return:返回一个 pandas.Panel
    """
    assert count, "count is required"
    from .finance_service import fundamentals_redundant_continuously_query_to_sql
    from .calendar_service import CalendarService

    trade_days = CalendarService.get_trade_days(end_date=end_date, count=count)
    sql = fundamentals_redundant_continuously_query_to_sql(query_object, trade_days)
    sql = remove_duplicated_tables(sql)
    df = JQDataClient.instance().get_fundamentals_continuously(sql=sql)
    df3 = df.copy()
    df3["multi"] = df["day"] + "_" + df["code"]
    df3 = df3.drop_duplicates("multi")
    del df3["multi"]
    df3 = df3.set_index(["day", "code"])
    pan = df3.to_panel()
    return pan


@assert_auth
def get_billboard_list(stock_list=None, start_date=None, end_date=None, count=None):
    """
    获取指定日期区间内的龙虎榜数据

    :param stock_list:一个股票代码的 list。 当值为 None 时， 返回指定日期的所有股票。
    :param start_date:开始日期
    :param end_date:结束日期
    :param count:交易日数量， 可以与 end_date 同时使用， 表示获取 end_date 前 count 个交易日的数据(含 end_date 当日)
    :return:回一个 pandas.DataFrame
    """
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    stock_list = convert_security(stock_list)
    return JQDataClient.instance().get_billboard_list(**locals())


@assert_auth
def get_locked_shares(stock_list=None, start_date=None, end_date=None, forward_count=None):
    """
    获取指定日期区间内的限售解禁数据

    :param stock_list:一个股票代码的 list
    :param start_date:开始日期
    :param end_date:结束日期
    :param forward_count:交易日数量， 可以与 start_date 同时使用， 表示获取 start_date 到 forward_count 个交易日区间的数据
    :return:
    """
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    stock_list = convert_security(stock_list)
    return JQDataClient.instance().get_locked_shares(**locals())


@assert_auth
def get_index_stocks(index_symbol, date=today()):
    """
    获取一个指数给定日期在平台可交易的成分股列表，请点击 https://www.joinquant.com/indexData 查看指数信息

    :param index_symbol 指数代码
    :param date: 查询日期, 一个字符串(格式类似’2015-10-15’)或者datetime.date/datetime.datetime对象, 可以是None, 使用默认日期.
    :return 股票代码的list
    """
    assert index_symbol, "index_symbol is required"
    date = to_date_str(date)
    return JQDataClient.instance().get_index_stocks(**locals())


@assert_auth
def get_industry_stocks(industry_code, date=today()):
    """
    获取在给定日期一个行业的所有股票，行业分类列表见 https://www.joinquant.com/data/dict/plateData

    :param industry_code 行业编码
    :param date 查询日期, 一个字符串(格式类似’2015-10-15’)或者datetime.date/datetime.datetime对象, 可以是None, 使用默认日期.
    :return 股票代码的list
    """
    assert industry_code, "industry_code is required"
    date = to_date_str(date)
    return JQDataClient.instance().get_industry_stocks(**locals())


@assert_auth
def get_industries(name=None):
    """
    按照行业分类获取行业列表
    :param name:行业代码
    :return:pandas.DataFrame, 各column分别为行业代码、行业名称、开始日期
    """
    assert name, "name is required"
    return JQDataClient.instance().get_industries(**locals())


@assert_auth
def get_concept_stocks(concept_code, date=today()):
    """
    获取在给定日期一个概念板块的所有股票，概念板块分类列表见 https://www.joinquant.com/data/dict/plateData

    :param concept_code 概念板块编码
    :param date: 查询日期, 一个字符串(格式类似’2015-10-15’)或者datetime.date/datetime.datetime对象, 可以是None, 使用默认日期.
    :return 股票代码的list
    """
    assert concept_code, "concept_code is required"
    date = to_date_str(date)
    return JQDataClient.instance().get_concept_stocks(**locals())


@assert_auth
def get_concepts():
    """
    获取概念板块
    :return:pandas.DataFrame, 各column分别为概念代码、概念名称、开始日期
    """
    return JQDataClient.instance().get_concepts(**locals())


@assert_auth
def get_all_securities(types=[], date=None):
    """
    获取平台支持的所有股票、基金、指数、期货信息

    :param types list: 用来过滤securities的类型, list元素可选: ‘stock’, ‘fund’, ‘index’, ‘futures’, ‘etf’, ‘lof’, ‘fja’, ‘fjb’. types为空时返回所有股票, 不包括基金,指数和期货
    :param date 日期, 一个字符串或者 datetime.datetime/datetime.date 对象, 用于获取某日期还在上市的股票信息. 默认值为 None, 表示获取所有日期的股票信息
    :return pandas.DataFrame
    """
    date = to_date_str(date)
    return JQDataClient.instance().get_all_securities(**locals())


@assert_auth
def get_security_info(code):
    """
    获取股票/基金/指数的信息

    :param code 证券代码
    :return Security
    """
    assert code, "code is required"
    result = JQDataClient.instance().get_security_info(**locals())
    if result:
        return Security(**result)


@assert_auth
def get_all_trade_days():
    """
    获取所有交易日

    :return 包含所有交易日的 numpy.ndarray, 每个元素为一个 datetime.date 类型.
    """
    data = JQDataClient.instance().get_all_trade_days()
    if str(data.dtype) != "object":
        data = data.astype(datetime.datetime)
    return data


@assert_auth
def get_trade_days(start_date=None, end_date=None, count=None):
    """
    获取指定日期范围内的所有交易日

    :return numpy.ndarray, 包含指定的 start_date 和 end_date, 默认返回至 datatime.date.today() 的所有交易日
    """
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    data =  JQDataClient.instance().get_trade_days(**locals())
    if str(data.dtype) != "object":
        data = data.astype(datetime.datetime)
    return data


@assert_auth
def get_money_flow(security_list, start_date=None, end_date=None, fields=None, count=None):
    """
    获取一只或者多只股票在一个时间段内的资金流向数据

    :param security_list 一只股票代码或者一个股票代码的 list
    :param start_date 开始日期, 与 count 二选一, 不可同时使用, 一个字符串或者 datetime.datetime/datetime.date 对象, 默认为平台提供的数据的最早日期
    :param end_date 结束日期, 一个字符串或者 datetime.date/datetime.datetime 对象, 默认为 datetime.date.today()
    :param count 数量, 与 start_date 二选一，不可同时使用, 必须大于 0. 表示返回 end_date 之前 count 个交易日的数据, 包含 end_date
    :param fields 字段名或者 list, 可选
    :return pandas.DataFrame
    """
    assert security_list, "security_list is required"
    security_list = convert_security(security_list)
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    return JQDataClient.instance().get_money_flow(**locals())


@assert_auth
def get_mtss(security_list, start_date=None, end_date=None, fields=None, count=None):
    """
    获取一只或者多只股票在一个时间段内的融资融券信息

    :param security_list 一只股票代码或者一个股票代码的 list
    :param start_date 开始日期, 与 count 二选一, 不可同时使用.
    :param end_date 结束日期, 一个字符串或者 datetime.date/datetime.datetime 对象, 默认为 datetime.date.today()
    :param count 数量, 与 start_date 二选一，不可同时使用, 必须大于 0. 表示返回 end_date 之前 count 个交易日的数据, 包含 end_date
    :param fields 字段名或者 list, 可选. 默认为 None, 表示取全部字段
    :return pandas.DataFrame
    """
    assert (not start_date) ^ (not count), "(start_date, count) only one param is required"
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    security_list = convert_security(security_list)
    return JQDataClient.instance().get_mtss(**locals())


@assert_auth
def get_margincash_stocks(dt=None):
    """
    返回上交所、深交所最近一次披露的的可融资标的列表的list
    :return: list
    """
    return JQDataClient.instance().get_margincash_stocks(**locals())


@assert_auth
def get_marginsec_stocks(dt=None):
    """
    返回上交所、深交所最近一次披露的的可融券标的列表的list
    :return:list
    """
    return JQDataClient.instance().get_marginsec_stocks(**locals())


@assert_auth
def get_future_contracts(underlying_symbol, dt=None):
    """
    获取某期货品种在策略当前日期的可交易合约标的列表

    :param security 期货合约品种，如 ‘AG’(白银)
    :return 某期货品种在策略当前日期的可交易合约标的列表
    """
    assert underlying_symbol, "underlying_symbol is required"
    dt = to_date_str(dt)
    return JQDataClient.instance().get_future_contracts(**locals())


@assert_auth
def get_dominant_future(underlying_symbol, dt=None):
    """
    获取主力合约对应的标的

    :param security 期货合约品种，如 ‘AG’(白银)
    :return 主力合约对应的期货合约
    """
    dt = to_date_str(dt)
    return JQDataClient.instance().get_dominant_future(**locals())


@assert_auth
def get_ticks(security, start_dt=None, end_dt=None, count=None, fields=None):
    """
    获取tick数据
    :param security: 股票or期货标的代码,仅限单只
    :param start_dt: 开始日期
    :param end_dt: 截止日期
    :param count: 统计个数
    :param fields: 期货：[time current high low volume money position a1_v a1_p b1_v b1_p]
                    股票：[time current high low volume money a1_v-a5_v a1_p-a5_p b1_v-b5_v b1_p-b5_p]
                    为None时，默认返回对应类型的所有字段
    :return:
    """
    start_dt = to_date_str(start_dt)
    end_dt = to_date_str(end_dt)
    return JQDataClient.instance().get_ticks(**locals())


@assert_auth
def get_baidu_factor(category=None, day=None, stock=None, province=None):
    """
    获取百度因子搜索量数据
    :param category:数据类别，中证800的数据类别为"csi800"
    :param stock: 一只股票或一个股票list。如果为空，则包含中证800所有的成分股。
    :param day:日期，date、datetime或字符串类型。如果day为空，则返回最新的数据。
    :param province:省份名称或省份代码，如北京或110000。如果为空，则返回PC端和手机端的数据汇总。
    如果不为空，则返回指定省份的数据。
    :return:
    """

    day = to_date_str(day)
    stock = normal_security_code(stock)
    return JQDataClient.instance().get_baidu_factor(**locals())


@assert_auth
def normalize_code(code):
    """
    归一化证券代码
    :param code 如000001
    :return 证券代码的全称 如000001.XSHE
    """
    return JQDataClient.instance().normalize_code(**locals())


@assert_auth
def get_factor_values(securities, factors, start_date=None, end_date=None, count=None):
    """
    获取因子数据
    返回字典类型
    :return:
    """

    securities = convert_security(securities)
    start_date = to_date_str(start_date)
    end_date = to_date_str(end_date)
    if (not count) and (not start_date):
            start_date = "2015-01-01"
    if count and start_date:
        raise ParamsError("(start_date, count) only one param is required")
    return JQDataClient.instance().get_factor_values(**locals())

@assert_auth
def get_index_weights(index_id, date=None):
    assert index_id, "index_id is required"
    date = to_date_str(date)
    return JQDataClient.instance().get_index_weights(**locals())


def read_file(path):
    """
    读取文件
    """
    with open(path, 'rb') as f:
        return f.read()


def write_file(path, content, append=False):
    """
    写入文件
    """
    if isinstance(content, six.text_type):
        content = content.encode('utf-8')
    with open(path, 'ab' if append else 'wb') as f:
        return f.write(content)


__all__ = ["get_price", "get_trade_days", "get_all_trade_days", "get_extras", "get_fundamentals_continuously",
            "get_index_stocks", "get_industry_stocks", "get_concept_stocks", "get_all_securities",
            "get_security_info", "get_money_flow", "get_locked_shares", "get_fundamentals", "get_mtss",
            "get_concepts", "get_industries", "get_margincash_stocks", "get_marginsec_stocks",
            "get_future_contracts", "get_dominant_future", "normalize_code", "get_baidu_factor",
            "get_billboard_list", "get_ticks", "read_file", "write_file", "get_factor_values", "get_index_weights"]



# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_INDUSTRY_AREA_AGR_OUTPUT_VALUE_YEAR(Base):
    """
    分地区农林牧渔业总产值表(年度)
    """
    __tablename__ = "MAC_INDUSTRY_AREA_AGR_OUTPUT_VALUE_YEAR"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    area_code = Column(String(20), nullable=False)
    area_name = Column(String(100), nullable=False)
    total = Column(DECIMAL(20, 4))
    farming = Column(DECIMAL(20, 4))
    forestry = Column(DECIMAL(20, 4))
    animal_husbandry = Column(DECIMAL(20, 4))
    fishery = Column(DECIMAL(20, 4))
    total_idx = Column(DECIMAL(10, 4))
    farming_idx = Column(DECIMAL(10, 4))
    forestry_idx = Column(DECIMAL(10, 4))
    animal_husbandry_idx = Column(DECIMAL(10, 4))
    fishery_idx = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addtime = Column(TIMESTAMP, default=datetime.datetime.now)
    modtime = Column(TIMESTAMP)

    @property
    def update_keys(self):
        return ["total",
                "farming",
                "forestry",
                "animal_husbandry",
                "fishery",
                "total_idx",
                "farming_idx",
                "forestry_idx",
                "animal_husbandry_idx",
                "fishery_idx"]

    @property
    def filter_keys(self):
        return ["stat_year", "area_code"]


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_INDUSTRY_CATEGORY_GROWTH(Base):
    """
    全国行业分类增长速度
    """

    __tablename__ = "MAC_INDUSTRY_CATEGORY_GROWTH"

    id = Column(Integer, primary_key=True)
    stat_month = Column(String(20), nullable=False)
    coal_mining_yoy = Column(DECIMAL(10, 4))
    oil_gas_mining_yoy = Column(DECIMAL(10, 4))
    black_metal_mining_yoy = Column(DECIMAL(10, 4))
    nonferrous_metal_mining_yoy = Column(DECIMAL(10, 4))
    nonmetal_mining_yoy = Column(DECIMAL(10, 4))
    mining_aid_yoy = Column(DECIMAL(10, 4))
    other_mining_yoy = Column(DECIMAL(10, 4))
    agro_food_yoy = Column(DECIMAL(10, 4))
    food_manu_yoy = Column(DECIMAL(10, 4))
    drink_manu_yoy = Column(DECIMAL(10, 4))
    tobacco_manu_yoy = Column(DECIMAL(10, 4))
    textile_yoy = Column(DECIMAL(10, 4))
    cloth_manu_yoy = Column(DECIMAL(10, 4))
    leather_manu_yoy = Column(DECIMAL(10, 4))
    wood_manu_yoy = Column(DECIMAL(10, 4))
    furniture_manu_yoy = Column(DECIMAL(10, 4))
    paper_manu_yoy = Column(DECIMAL(10, 4))
    print_yoy = Column(DECIMAL(10, 4))
    refinery_yoy = Column(DECIMAL(10, 4))
    chemical_manu_yoy = Column(DECIMAL(10, 4))
    medical_manu_yoy = Column(DECIMAL(10, 4))
    chemical_fiber_yoy = Column(DECIMAL(10, 4))
    rubber_manu_yoy = Column(DECIMAL(10, 4))
    nonmetal_manu_yoy = Column(DECIMAL(10, 4))
    black_metal_manu_yoy = Column(DECIMAL(10, 4))
    nonferrous_metal_manu_yoy = Column(DECIMAL(10, 4))
    metal_product_manu_yoy = Column(DECIMAL(10, 4))
    general_equipment_manu_yoy = Column(DECIMAL(10, 4))
    professional_equipment_manu_yoy = Column(DECIMAL(10, 4))
    car_manu_yoy = Column(DECIMAL(10, 4))
    transport_manu_yoy = Column(DECIMAL(10, 4))
    electrical_equipment_manu_yoy = Column(DECIMAL(10, 4))
    communication_equipment_manu_yoy = Column(DECIMAL(10, 4))
    meter_manu_yoy = Column(DECIMAL(10, 4))
    other_manu_yoy = Column(DECIMAL(10, 4))
    reuse_manu_yoy = Column(DECIMAL(10, 4))
    repair_manu_yoy = Column(DECIMAL(10, 4))
    power_yoy = Column(DECIMAL(10, 4))
    gas_yoy = Column(DECIMAL(10, 4))
    water_yoy = Column(DECIMAL(10, 4))
    coal_mining_acc = Column(DECIMAL(10, 4))
    oil_gas_mining_acc = Column(DECIMAL(10, 4))
    black_metal_mining_acc = Column(DECIMAL(10, 4))
    nonferrous_metal_mining_acc = Column(DECIMAL(10, 4))
    nonmetal_mining_acc = Column(DECIMAL(10, 4))
    mining_aid_acc = Column(DECIMAL(10, 4))
    other_mining_acc = Column(DECIMAL(10, 4))
    agro_food_acc = Column(DECIMAL(10, 4))
    food_manu_acc = Column(DECIMAL(10, 4))
    drink_manu_acc = Column(DECIMAL(10, 4))
    tobacco_manu_acc = Column(DECIMAL(10, 4))
    textile_acc = Column(DECIMAL(10, 4))
    cloth_manu_acc = Column(DECIMAL(10, 4))
    leather_manu_acc = Column(DECIMAL(10, 4))
    wood_manu_acc = Column(DECIMAL(10, 4))
    furniture_manu_acc = Column(DECIMAL(10, 4))
    paper_manu_acc = Column(DECIMAL(10, 4))
    print_acc = Column(DECIMAL(10, 4))
    refinery_acc = Column(DECIMAL(10, 4))
    chemical_manu_acc = Column(DECIMAL(10, 4))
    medical_manu_acc = Column(DECIMAL(10, 4))
    chemical_fiber_acc = Column(DECIMAL(10, 4))
    rubber_manu_acc = Column(DECIMAL(10, 4))
    nonmetal_manu_acc = Column(DECIMAL(10, 4))
    black_metal_manu_acc = Column(DECIMAL(10, 4))
    nonferrous_metal_manu_acc = Column(DECIMAL(10, 4))
    metal_product_manu_acc = Column(DECIMAL(10, 4))
    general_equipment_manu_acc = Column(DECIMAL(10, 4))
    professional_equipment_manu_acc = Column(DECIMAL(10, 4))
    car_manu_acc = Column(DECIMAL(10, 4))
    transport_manu_acc = Column(DECIMAL(10, 4))
    electrical_equipment_manu_acc = Column(DECIMAL(10, 4))
    communication_equipment_manu_acc = Column(DECIMAL(10, 4))
    meter_manu_acc = Column(DECIMAL(10, 4))
    other_manu_acc = Column(DECIMAL(10, 4))
    reuse_manu_acc = Column(DECIMAL(10, 4))
    repair_manu_acc = Column(DECIMAL(10, 4))
    power_acc = Column(DECIMAL(10, 4))
    gas_acc = Column(DECIMAL(10, 4))
    water_acc = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_INDUSTRY_AGR_PRODUCT_IDX_QUARTER(Base):
    """
    全国农产品生产价格指数表(季度)
    """
    __tablename__ = "MAC_INDUSTRY_AGR_PRODUCT_IDX_QUARTER"

    id = Column(Integer, primary_key=True)
    stat_quarter = Column(String(20), nullable=False)
    agricultural_products = Column(DECIMAL(20, 4))
    crop_products = Column(DECIMAL(20, 4))
    food = Column(DECIMAL(20, 4))
    grain = Column(DECIMAL(20, 4))
    wheat = Column(DECIMAL(20, 4))
    rice = Column(DECIMAL(20, 4))
    corn = Column(DECIMAL(20, 4))
    bean = Column(DECIMAL(20, 4))
    soybean = Column(DECIMAL(20, 4))
    potato = Column(DECIMAL(20, 4))
    oil_plants = Column(DECIMAL(20, 4))
    cotton = Column(DECIMAL(20, 4))
    sugar = Column(DECIMAL(20, 4))
    tobacco = Column(DECIMAL(20, 4))
    vegetable = Column(DECIMAL(20, 4))
    fruit = Column(DECIMAL(20, 4))
    tea = Column(DECIMAL(20, 4))
    forestry_products = Column(DECIMAL(20, 4))
    wood = Column(DECIMAL(20, 4))
    bamboo = Column(DECIMAL(20, 4))
    pectin = Column(DECIMAL(20, 4))
    animal_husbandry_products = Column(DECIMAL(20, 4))
    pig = Column(DECIMAL(20, 4))
    cow = Column(DECIMAL(20, 4))
    sheep = Column(DECIMAL(20, 4))
    poultry = Column(DECIMAL(20, 4))
    egg = Column(DECIMAL(20, 4))
    milk = Column(DECIMAL(20, 4))
    wool = Column(DECIMAL(20, 4))
    fishery = Column(DECIMAL(20, 4))
    marine_fishery_products = Column(DECIMAL(20, 4))
    marine_farm_products = Column(DECIMAL(20, 4))
    fresh_fishery_products = Column(DECIMAL(20, 4))
    fresh_farm_products = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addtime = Column(TIMESTAMP, default=datetime.datetime.now)
    modtime = Column(TIMESTAMP)

    @property
    def update_keys(self):
        return ["agricultural_products",
                "crop_products",
                "food",
                "grain",
                "wheat",
                "rice",
                "corn",
                "bean",
                "soybean",
                "potato",
                "oil_plants",
                "cotton",
                "sugar",
                "tobacco",
                "vegetable",
                "fruit",
                "tea",
                "forestry_products",
                "wood",
                "bamboo",
                "pectin",
                "animal_husbandry_products",
                "pig",
                "cow",
                "sheep",
                "poultry",
                "egg",
                "milk",
                "wool",
                "fishery",
                "marine_fishery_products",
                "marine_farm_products",
                "fresh_fishery_products",
                "fresh_farm_products"]

    @property
    def filter_keys(self):
        return ["stat_quarter"]


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_INSURANCE_PREMIUM_YEAR(Base):
    """
    保险公司保费金额表（年）
    """
    __tablename__ = "MAC_INSURANCE_PREMIUM_YEAR"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    total_income = Column(DECIMAL(20, 4))
    property_income = Column(DECIMAL(20, 4))
    enterprise = Column(DECIMAL(20, 4))
    family = Column(DECIMAL(20, 4))
    vehicle = Column(DECIMAL(20, 4))
    project = Column(DECIMAL(20, 4))
    liability = Column(DECIMAL(20, 4))
    credit = Column(DECIMAL(20, 4))
    guarantee = Column(DECIMAL(20, 4))
    ship = Column(DECIMAL(20, 4))
    freight = Column(DECIMAL(20, 4))
    special_risk = Column(DECIMAL(20, 4))
    farm = Column(DECIMAL(20, 4))
    property_health = Column(DECIMAL(20, 4))
    property_accident = Column(DECIMAL(20, 4))
    other = Column(DECIMAL(20, 4))
    personal_income = Column(DECIMAL(20, 4))
    life = Column(DECIMAL(20, 4))
    health = Column(DECIMAL(20, 4))
    accident = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addtime = Column(TIMESTAMP, default=datetime.datetime.now)
    modtime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_POP_FERTILITY_RATE(Base):
    """
    育龄妇女分年龄、孩次的生育状况表
    """
    __tablename__ = "MAC_POP_FERTILITY_RATE"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    age = Column(String(30), nullable=False)
    rate = Column(DECIMAL(10, 8))
    population = Column(DECIMAL(20, 4))
    births = Column(DECIMAL(20, 4))
    first_births = Column(DECIMAL(20, 4))
    second_births = Column(DECIMAL(20, 4))
    third_births = Column(DECIMAL(20, 4))
    fertility_rate = Column(DECIMAL(20, 4))
    first_fertility_rate = Column(DECIMAL(10, 4))
    second_fertility_rate = Column(DECIMAL(10, 4))
    third_fertility_rate = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addtime = Column(TIMESTAMP, default=datetime.datetime.now)
    modtime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_AREA_RURAL_NET_INCOME_YEAR(Base):
    """
    各地区按来源分农村居民家庭人均纯收入(年度)
    """
    __tablename__ = "MAC_AREA_RURAL_NET_INCOME_YEAR"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    area_code = Column(String(20), nullable=False)
    area_name = Column(String(100), nullable=False)
    income = Column(DECIMAL(20, 4))
    wage_income = Column(DECIMAL(20, 4))
    business_income = Column(DECIMAL(20, 4))
    property_income = Column(DECIMAL(20, 4))
    transfer_income = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addtime = Column(TIMESTAMP, default=datetime.datetime.now)
    modtime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_AREA_FISCAL_EXPENSE_YEAR(Base):
    """
    各地区财政支出表（年度）
    """
    __tablename__ = "MAC_AREA_FISCAL_EXPENSE_YEAR"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    area_code = Column(String(20), nullable=False)
    area_name = Column(String(100), nullable=False)
    general_budget = Column(DECIMAL(20, 4))
    public_sevice = Column(DECIMAL(20, 4))
    diplomacy = Column(DECIMAL(20, 4))
    defence = Column(DECIMAL(20, 4))
    public_security = Column(DECIMAL(20, 4))
    education = Column(DECIMAL(20, 4))
    science_technology = Column(DECIMAL(20, 4))
    cultural_sports_media = Column(DECIMAL(20, 4))
    social_security_employment = Column(DECIMAL(20, 4))
    public_health = Column(DECIMAL(20, 4))
    environmental_protection = Column(DECIMAL(20, 4))
    community_affairs = Column(DECIMAL(20, 4))
    agriculture_forestry = Column(DECIMAL(20, 4))
    transportation = Column(DECIMAL(20, 4))
    resource_exploration_power = Column(DECIMAL(20, 4))
    business_service = Column(DECIMAL(20, 4))
    financial_supervision = Column(DECIMAL(20, 4))
    earthquake_reconstruction = Column(DECIMAL(20, 4))
    land_resource_meteorology = Column(DECIMAL(20, 4))
    housing_security = Column(DECIMAL(20, 4))
    material_reserve_management = Column(DECIMAL(20, 4))
    debt_interest = Column(DECIMAL(20, 4))
    other = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding=utf-8
from .utils import *
from .api import *
import thriftpy
from thriftpy.rpc import make_client
import msgpack
import time
from os import path
import platform
import sys
import threading
import socket


thrift_path = path.join(sys.modules["ROOT_DIR"], "jqdata.thrift")
thrift_path = path.abspath(thrift_path)
module_name = path.splitext(path.basename(thrift_path))[0]
thrift = None
with open(thrift_path) as f:
    thrift = thriftpy.load_fp(f, "jqdata_thrift")


class JQDataClient(object):

    _threading_local = threading.local()
    _auth_params = {}

    @classmethod
    def instance(cls):
        _instance = getattr(cls._threading_local, '_instance', None)
        if _instance is None:
            if cls._auth_params:
                _instance = JQDataClient(**cls._auth_params)
            cls._threading_local._instance = _instance
        return _instance

    def __init__(self, host, port, username="", password="", retry_cnt=30):
        assert host, "host is required"
        assert port, "port is required"
        assert username, "username is required"
        assert password, "password is required"
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.client = None
        self.inited = False
        self.retry_cnt = retry_cnt
        self.not_auth = True

    @classmethod
    def set_auth_params(cls, **params):
        cls._auth_params = params
        cls.instance().ensure_auth()

    def ensure_auth(self):
        if not self.inited:
            if not self.username or self.username == "":
                raise RuntimeError("not inited")
            self.client = make_client(thrift.JqDataService, self.host, self.port)
            self.inited = True
            response = self.client.auth(self.username, self.password)
            if not response.status:
                self._threading_local._instance = None
                raise self.get_error(response)
            else:
                if self.not_auth:
                    print("auth success")
                    self.not_auth = False

    def _reset(self):
        if self.client:
            self.client.close()
            self.client = None
        self.inited = False

    def get_error(self, response):
        err = None
        if six.PY2:
            system = platform.system().lower()
            if system == "windows":
                err = Exception(response.error.encode("gbk"))
            else:
                err = Exception(response.error.encode("utf-8"))
        else:
            err = Exception(response.error)
        return err

    def __call__(self, method, **kwargs):
        request = thrift.St_Query_Req()
        request.method_name = method
        request.params = msgpack.packb(kwargs)
        import tempfile

        err, result = None, None
        for idx in range(self.retry_cnt):
            d = tempfile.gettempdir()
            import os, random, string
            name2 = ''.join(random.sample(string.ascii_letters + string.digits, 10))
            file = open(os.path.join(d, name2), "w+b")
            try:
                self.ensure_auth()
                response = self.client.query(request)
                if response.status:
                    buffer = response.msg
                    if six.PY3:
                        if type(buffer) is str:
                            buffer = bytes(buffer, "ascii")
                    file.write(buffer)
                    file.seek(0)
                    result = pd.read_pickle(file.name)
                else:
                    err = self.get_error(response)
                break
            except KeyboardInterrupt as e:
                self._reset()
                err = e
                raise
            except (thriftpy.transport.TTransportException, socket.error) as e:
                self._reset()
                err = e
                time.sleep(idx * 2)
                continue
            except Exception as e:
                self._reset()
                err = e
                break
            finally:
                if os.path.exists(file.name):
                    file.close()
                    os.unlink(file.name)

        if result is None:
            if isinstance(err, Exception):
                raise err

        return result

    def __getattr__(self, method):
        return lambda **kwargs: self(method, **kwargs)





# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_AREA_FIXED_INVESTMENT(Base):
    """
    分地区固定资产投资情况
    """
    __tablename__ = "MAC_AREA_FIXED_INVESTMENT"

    id = Column(Integer, primary_key=True)
    stat_month = Column(String(20), nullable=False)
    area_code = Column(String(20), nullable=False)
    area_name = Column(String(100), nullable=False)
    fixed_assets_investment = Column(DECIMAL(20, 4))
    construct_install = Column(DECIMAL(20, 4))
    equipment_purchase = Column(DECIMAL(20, 4))
    other_expense = Column(DECIMAL(20, 4))
    resident_construct = Column(DECIMAL(20, 4))
    construct_area = Column(DECIMAL(20, 4))
    resident_complete = Column(DECIMAL(20, 4))
    construct_num = Column(Integer)
    new_construct_num = Column(Integer)
    fixed_assets_investment_yoy = Column(DECIMAL(10, 4))
    construct_install_yoy = Column(DECIMAL(10, 4))
    equipment_purchase_yoy = Column(DECIMAL(10, 4))
    other_expense_yoy = Column(DECIMAL(10, 4))
    resident_construct_yoy = Column(DECIMAL(10, 4))
    construct_area_yoy = Column(DECIMAL(10, 4))
    resident_complete_yoy = Column(DECIMAL(10, 4))
    construct_num_yoy = Column(DECIMAL(10, 4))
    new_construct_num_yoy = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addtime = Column(TIMESTAMP, default=datetime.datetime.now)
    modtime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_CURRENCY_STATE_YEAR(Base):
    """
    货币当局资产负债表(年度)
    """

    __tablename__ = "MAC_CURRENCY_STATE_YEAR"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    total_assets = Column(DECIMAL(20, 4))
    foreign_assets = Column(DECIMAL(20, 4))
    foreign_exchange = Column(DECIMAL(20, 4))
    money_gold = Column(DECIMAL(20, 4))
    other_foreign_assets = Column(DECIMAL(20, 4))
    government_claim = Column(DECIMAL(20, 4))
    bank_claim = Column(DECIMAL(20, 4))
    other_finance_claim = Column(DECIMAL(20, 4))
    non_finance_claim = Column(DECIMAL(20, 4))
    other_assets = Column(DECIMAL(20, 4))
    total_liability = Column(DECIMAL(20, 4))
    reserve_money = Column(DECIMAL(20, 4))
    currency_issue = Column(DECIMAL(20, 4))
    finance_deposit = Column(DECIMAL(20, 4))
    bank_deposit = Column(DECIMAL(20, 4))
    other_finance_deposit = Column(DECIMAL(20, 4))
    non_reserve_finance_deposit = Column(DECIMAL(20, 4))
    bond_issue = Column(DECIMAL(20, 4))
    foreign_liability = Column(DECIMAL(20, 4))
    government_deposit = Column(DECIMAL(20, 4))
    owned_capital = Column(DECIMAL(20, 4))
    other_liability = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_INDUSTRY_ESTATE_INVEST_MONTH(Base):
    """
    房地产开发投资情况表(月度累计)
    """
    __tablename__ = "MAC_INDUSTRY_ESTATE_INVEST_MONTH"

    id = Column(Integer, primary_key=True)
    stat_month = Column(String(20), nullable=False)
    invest = Column(DECIMAL(20, 4))
    invest_yoy = Column(DECIMAL(10, 4))
    auxiliary_project = Column(DECIMAL(20, 4))
    auxiliary_project_yoy = Column(DECIMAL(10, 4))
    resident = Column(DECIMAL(20, 4))
    resident_yoy = Column(DECIMAL(10, 4))
    below90_house = Column(DECIMAL(20, 4))
    below90_house_yoy = Column(DECIMAL(10, 4))
    above144_house = Column(DECIMAL(20, 4))
    above144_house_yoy = Column(DECIMAL(10, 4))
    villa_flat = Column(DECIMAL(20, 4))
    villa_flat_yoy = Column(DECIMAL(10, 4))
    office = Column(DECIMAL(20, 4))
    office_yoy = Column(DECIMAL(10, 4))
    business = Column(DECIMAL(20, 4))
    business_yoy = Column(DECIMAL(10, 4))
    other_house = Column(DECIMAL(20, 4))
    other_house_yoy = Column(DECIMAL(10, 4))
    construct = Column(DECIMAL(20, 4))
    construct_yoy = Column(DECIMAL(10, 4))
    install = Column(DECIMAL(20, 4))
    install_yoy = Column(DECIMAL(10, 4))
    equipment_purchase = Column(DECIMAL(20, 4))
    equipment_purchase_yoy = Column(DECIMAL(10, 4))
    other_expense = Column(DECIMAL(20, 4))
    other_expense_yoy = Column(DECIMAL(10, 4))
    land_purchase = Column(DECIMAL(20, 4))
    land_purchase_yoy = Column(DECIMAL(10, 4))
    plan_invest = Column(DECIMAL(20, 4))
    plan_invest_yoy = Column(DECIMAL(10, 4))
    new_fixed_assets = Column(DECIMAL(20, 4))
    new_fixed_assets_yoy = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addtime = Column(TIMESTAMP, default=datetime.datetime.now)
    modtime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_AREA_SALE_MARKET(Base):
    """
    分地区亿元以上商品交易市场基本情况
    """

    __tablename__ = "MAC_AREA_SALE_MARKET"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    area_code = Column(String(20), nullable=False)
    area_name = Column(String(100), nullable=False)
    market_num = Column(DECIMAL(20, 4))
    stall_num = Column(DECIMAL(20, 4))
    operation_area = Column(DECIMAL(20, 4))
    turnover = Column(DECIMAL(20, 4))
    turnover_wholesale = Column(DECIMAL(20, 4))
    turnover_retail = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_AREA_GDP_QUARTER(Base):
    """
    分地区国内生产总值表（季度）
    """
    __tablename__ = "MAC_AREA_GDP_QUARTER"

    id = Column(Integer, primary_key=True)
    stat_quarter = Column(String(14), nullable=False)
    area_code = Column(String(20), nullable=False)
    area_name = Column(String(100), nullable=False)
    gdp_acc = Column(DECIMAL(20, 4))
    gdp_primary_acc = Column(DECIMAL(20, 4))
    gdp_secondary_acc = Column(DECIMAL(20, 4))
    gdp_tertiary_acc = Column(DECIMAL(20, 4))
    gdp_sin = Column(DECIMAL(20, 4))
    gdp_primary_sin = Column(DECIMAL(20, 4))
    gdp_secondary_sin = Column(DECIMAL(20, 4))
    gdp_tertiary_sin = Column(DECIMAL(20, 4))
    gdp_yoy_acc = Column(DECIMAL(10, 4))
    gdp_primary_yoy_acc = Column(DECIMAL(10, 4))
    gdp_secondary_yoy_acc = Column(DECIMAL(10, 4))
    gdp_tertiary_yoy_acc = Column(DECIMAL(10, 4))
    gdp_yoy_sin = Column(DECIMAL(10, 4))
    gdp_primary_yoy_sin = Column(DECIMAL(10, 4))
    gdp_secondary_yoy_sin = Column(DECIMAL(10, 4))
    gdp_tertiary_yoy_sin = Column(DECIMAL(10, 4))
    gdp_mom = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding=utf-8

"""
宏观经济数据
"""
from .utils import *
import sys


class Macro(object):
    RESULT_ROWS_LIMIT = 3000

    def __init__(self, disable_join=False):
        self.__disable_join = True
        self.__table_names = self.__load_table_names()
        for name in self.__table_names:
            setattr(self, name, self.__load_table_class(name))

    def __load_table_names(self):
        import os
        tables_dir = os.path.join(sys.modules["ROOT_DIR"], 'macro_tables')
        names = []
        if os.path.exists(tables_dir):
            for table_file in os.listdir(tables_dir):
                if table_file.endswith('.py') and not table_file.startswith('__'):
                    names.append(table_file[:-3])
        return names

    @assert_auth
    def run_query(self, query_object):
        from .client import JQDataClient
        if self.__disable_join:
            check_no_join(query_object)

        limit = self.RESULT_ROWS_LIMIT
        if query_object._limit:
            limit = min(limit, query_object._limit)
        query_object = query_object.limit(limit)

        sql = compile_query(query_object)
        df = JQDataClient.instance().macro_query(sql=sql)
        return df

    def __load_table_class(self, table_name):
        table_module = __import__('jqdatasdk.macro_tables.' + table_name, fromlist=[table_name])
        return getattr(table_module, table_name)

    # def __getattribute__(self, key):
    #     v = object.__getattribute__(self, key)
    #     if v is None:
    #         if key in self.__table_names:
    #             v = self.__load_table_class(key)
    #             setattr(self, key, v)
    #     return v


macro = Macro()





# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_AREA_REGISTERED_WAGE_YEAR(Base):
    """
    分地区按注册类型分城镇单位就业人员工资情况表(年度)
    """

    __tablename__ = "MAC_AREA_REGISTERED_WAGE_YEAR"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    area_code = Column(String(20), nullable=False)
    area_name = Column(String(100), nullable=False)
    wage_avg = Column(DECIMAL(20, 4))
    wage_avg_state_owned = Column(DECIMAL(20, 4))
    wage_avg_collective = Column(DECIMAL(20, 4))
    wage_avg_stock_cooperate = Column(DECIMAL(20, 4))
    wage_avg_joint_ownership = Column(DECIMAL(20, 4))
    wage_avg_limited = Column(DECIMAL(20, 4))
    wage_avg_stock = Column(DECIMAL(20, 4))
    wage_avg_private = Column(DECIMAL(20, 4))
    wage_avg_hkmt = Column(DECIMAL(20, 4))
    wage_avg_foreign = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_SOCIAL_SCALE_FINANCE(Base):
    """
    社会融资规模及构成
    """

    __tablename__ = "MAC_SOCIAL_SCALE_FINANCE"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    social_finance_scale = Column(DECIMAL(20, 4))
    rmb_loan = Column(DECIMAL(20, 4))
    foreign_loan = Column(DECIMAL(20, 4))
    entrust_loan = Column(DECIMAL(20, 4))
    trust_loan = Column(DECIMAL(20, 4))
    out_fulfilled_scale = Column(DECIMAL(20, 4))
    corporate_bond_scale = Column(DECIMAL(20, 4))
    non_finance_scale = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_STK_MARKET(Base):
    """
    证券市场基本情况(年度)
    """

    __tablename__ = "MAC_STK_MARKET"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    public_company = Column(DECIMAL(20, 4))
    public_b_company = Column(DECIMAL(20, 4))
    public_h_company = Column(DECIMAL(20, 4))
    total_share = Column(DECIMAL(20, 4))
    flow_share = Column(DECIMAL(20, 4))
    total_value = Column(DECIMAL(20, 4))
    flow_value = Column(DECIMAL(20, 4))
    total_trade_volume = Column(DECIMAL(20, 4))
    total_trade_amount = Column(DECIMAL(20, 4))
    xshg_close = Column(DECIMAL(20, 4))
    xshe_close = Column(DECIMAL(20, 4))
    account_num = Column(DECIMAL(20, 4))
    xshg_avg_pe = Column(DECIMAL(10, 4))
    xshe_avg_pe = Column(DECIMAL(10, 4))
    xshg_avg_turnover = Column(DECIMAL(10, 4))
    xshe_avg_turnover = Column(DECIMAL(10, 4))
    treasury_bond_issue = Column(DECIMAL(20, 4))
    company_bond_issue = Column(DECIMAL(20, 4))
    bond_amount = Column(DECIMAL(20, 4))
    treasury_bond_spot_amount = Column(DECIMAL(20, 4))
    treasury_bond_repurchase_amount = Column(DECIMAL(20, 4))
    security_fund_num = Column(DECIMAL(20, 4))
    security_fund_cap = Column(DECIMAL(20, 4))
    security_fund_amount = Column(DECIMAL(20, 4))
    future_volume = Column(DECIMAL(20, 4))
    future_amount = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_CREDIT_BALANCE_YEAR(Base):
    """
    金融机构信贷资金平衡表(年度)
    """

    __tablename__ = "MAC_CREDIT_BALANCE_YEAR"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    total_source = Column(DECIMAL(20, 4))
    total_deposit = Column(DECIMAL(20, 4))
    corporate_deposit = Column(DECIMAL(20, 4))
    personal_deposit = Column(DECIMAL(20, 4))
    fiscal_deposit = Column(DECIMAL(20, 4))
    temporary_deposit = Column(DECIMAL(20, 4))
    designated_deposit = Column(DECIMAL(20, 4))
    finance_bond = Column(DECIMAL(20, 4))
    m0 = Column(DECIMAL(20, 4))
    finance_liability = Column(DECIMAL(20, 4))
    total_use = Column(DECIMAL(20, 4))
    total_loan = Column(DECIMAL(20, 4))
    domestic_loan = Column(DECIMAL(20, 4))
    short_term_loan = Column(DECIMAL(20, 4))
    medium_long_term_loan = Column(DECIMAL(20, 4))
    finance_lease = Column(DECIMAL(20, 4))
    finance_bill = Column(DECIMAL(20, 4))
    advance = Column(DECIMAL(20, 4))
    oversea_loan = Column(DECIMAL(20, 4))
    security = Column(DECIMAL(20, 4))
    othershare = Column(DECIMAL(20, 4))
    bullion_purchase = Column(DECIMAL(20, 4))
    forex_purchase = Column(DECIMAL(20, 4))
    finance_assets = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_AREA_FISCAL_REVENUE_YEAR(Base):
    """
    各地区财政收入表（年度）
    """
    __tablename__ = "MAC_AREA_FISCAL_REVENUE_YEAR"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    area_code = Column(String(20), nullable=False)
    area_name = Column(String(100), nullable=False)
    general_budget = Column(DECIMAL(20, 4))
    tax = Column(DECIMAL(20, 4))
    add_value_tax = Column(DECIMAL(20, 4))
    business_tax = Column(DECIMAL(20, 4))
    corporate_income_tax = Column(DECIMAL(20, 4))
    individual_income_tax = Column(DECIMAL(20, 4))
    resource_tax = Column(DECIMAL(20, 4))
    adjustment_tax = Column(DECIMAL(20, 4))
    maintenance_construction_tax = Column(DECIMAL(20, 4))
    house_property_tax = Column(DECIMAL(20, 4))
    stamp_tax = Column(DECIMAL(20, 4))
    land_tax = Column(DECIMAL(20, 4))
    land_increment_tax = Column(DECIMAL(20, 4))
    vehicle_vessel_tax = Column(DECIMAL(20, 4))
    land_occupation_tax = Column(DECIMAL(20, 4))
    contract_tax = Column(DECIMAL(20, 4))
    tobacco_tax = Column(DECIMAL(20, 4))
    other_tax = Column(DECIMAL(20, 4))
    non_tax = Column(DECIMAL(20, 4))
    special = Column(DECIMAL(20, 4))
    administration = Column(DECIMAL(20, 4))
    punishment = Column(DECIMAL(20, 4))
    capital_operation = Column(DECIMAL(20, 4))
    asset_use = Column(DECIMAL(20, 4))
    other_non_tax = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_OTHER_DEPOSIT(Base):
    """
    其他存款性公司资产负债表
    """
    __tablename__ = "MAC_OTHER_DEPOSIT"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    total_assets = Column(DECIMAL(20, 4))
    foreign_assets = Column(DECIMAL(20, 4))
    reserve_assets = Column(DECIMAL(20, 4))
    reserve_deposit = Column(DECIMAL(20, 4))
    cash_in_vault = Column(DECIMAL(20, 4))
    government_claim = Column(DECIMAL(20, 4))
    central_bank_claim = Column(DECIMAL(20, 4))
    other_claim = Column(DECIMAL(20, 4))
    other_finance_claim = Column(DECIMAL(20, 4))
    non_finance_claim = Column(DECIMAL(20, 4))
    other_resident_claim = Column(DECIMAL(20, 4))
    other_assets = Column(DECIMAL(20, 4))
    total_liability = Column(DECIMAL(20, 4))
    non_finance_liability = Column(DECIMAL(20, 4))
    non_finance_include_broad_money = Column(DECIMAL(20, 4))
    corporate_demand_deposit = Column(DECIMAL(20, 4))
    corporate_time_deposit = Column(DECIMAL(20, 4))
    personal_deposit = Column(DECIMAL(20, 4))
    exclude_broad_money = Column(DECIMAL(20, 4))
    transfer_deposit = Column(DECIMAL(20, 4))
    other_deposit = Column(DECIMAL(20, 4))
    other_non_finance_liability = Column(DECIMAL(20, 4))
    central_bank_liability = Column(DECIMAL(20, 4))
    other_deposit_liability = Column(DECIMAL(20, 4))
    other_finance_liability = Column(DECIMAL(20, 4))
    include_broad_money = Column(DECIMAL(20, 4))
    foreign_liability = Column(DECIMAL(20, 4))
    bond_issue = Column(DECIMAL(20, 4))
    paid_in_capital = Column(DECIMAL(20, 4))
    other_liability = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_FIXED_INVESTMENT(Base):
    """
    固定资产投资情况
    """
    __tablename__ = "MAC_FIXED_INVESTMENT"

    id = Column(Integer, primary_key=True)
    stat_month = Column(String(20), nullable=False)
    fixed_assets_investment = Column(DECIMAL(20, 4))
    state_owned = Column(DECIMAL(20, 4))
    real_estate = Column(DECIMAL(20, 4))
    primary = Column(DECIMAL(20, 4))
    secondary = Column(DECIMAL(20, 4))
    tertiary = Column(DECIMAL(20, 4))
    centre_project = Column(DECIMAL(20, 4))
    local_project = Column(DECIMAL(20, 4))
    new_construct = Column(DECIMAL(20, 4))
    expand = Column(DECIMAL(20, 4))
    reconstruct = Column(DECIMAL(20, 4))
    construct_install = Column(DECIMAL(20, 4))
    equipment_purchase = Column(DECIMAL(20, 4))
    other_expense = Column(DECIMAL(20, 4))
    construct_area = Column(DECIMAL(20, 4))
    resident_complete_area = Column(DECIMAL(20, 4))
    new_fixed_assets = Column(DECIMAL(20, 4))
    fixed_assets_investment_yoy = Column(DECIMAL(10, 4))
    state_owned_yoy = Column(DECIMAL(10, 4))
    real_estate_yoy = Column(DECIMAL(10, 4))
    primary_yoy = Column(DECIMAL(10, 4))
    secondary_yoy = Column(DECIMAL(10, 4))
    tertiary_yoy = Column(DECIMAL(10, 4))
    centre_project_yoy = Column(DECIMAL(10, 4))
    local_project_yoy = Column(DECIMAL(10, 4))
    new_construct_yoy = Column(DECIMAL(10, 4))
    expand_yoy = Column(DECIMAL(10, 4))
    reconstruct_yoy = Column(DECIMAL(10, 4))
    construct_install_yoy = Column(DECIMAL(10, 4))
    equipment_purchase_yoy = Column(DECIMAL(10, 4))
    other_expense_yoy = Column(DECIMAL(10, 4))
    construct_area_yoy = Column(DECIMAL(10, 4))
    resident_complete_area_yoy = Column(DECIMAL(10, 4))
    new_fixed_assets_yoy = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_STK_TRADE(Base):
    """
    股票市场统计表
    """

    __tablename__ = "MAC_STK_TRADE"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    public_company = Column(DECIMAL(20, 4))
    stock_num = Column(DECIMAL(20, 4))
    a_stock_num = Column(DECIMAL(20, 4))
    b_stock_num = Column(DECIMAL(20, 4))
    total_share = Column(DECIMAL(20, 4))
    a_total_share = Column(DECIMAL(20, 4))
    b_total_share = Column(DECIMAL(20, 4))
    circulating_share = Column(DECIMAL(20, 4))
    a_circulating_share = Column(DECIMAL(20, 4))
    b_circulating_share = Column(DECIMAL(20, 4))
    total_value = Column(DECIMAL(20, 4))
    a_total_value = Column(DECIMAL(20, 4))
    b_total_value = Column(DECIMAL(20, 4))
    circulation_value = Column(DECIMAL(20, 4))
    a_circulation_value = Column(DECIMAL(20, 4))
    b_circulation_value = Column(DECIMAL(20, 4))
    amount = Column(DECIMAL(20, 4))
    a_amount = Column(DECIMAL(20, 4))
    b_amount = Column(DECIMAL(20, 4))
    volume = Column(DECIMAL(20, 4))
    a_volume = Column(DECIMAL(20, 4))
    b_volume = Column(DECIMAL(20, 4))
    sh_composite_index_high = Column(DECIMAL(20, 4))
    sh_composite_index_low = Column(DECIMAL(20, 4))
    sh_composite_index_close = Column(DECIMAL(20, 4))
    sz_composite_index_high = Column(DECIMAL(20, 4))
    sz_composite_index_low = Column(DECIMAL(20, 4))
    sz_composite_index_close = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_FOREIGN_CAPITAL_MONTH(Base):
    """
    利用外资情况表（月度）
    """
    __tablename__ = "MAC_FOREIGN_CAPITAL_MONTH"

    id = Column(Integer, primary_key=True)
    stat_month = Column(String(20), nullable=False)
    num_acc = Column(DECIMAL(20, 4))
    num_acc_yoy = Column(DECIMAL(10, 4))
    joint_num_acc = Column(DECIMAL(20, 4))
    joint_num_acc_yoy = Column(DECIMAL(10, 4))
    cooperative_num_acc = Column(DECIMAL(20, 4))
    cooperative_num_acc_yoy = Column(DECIMAL(10, 4))
    foreign_num_acc = Column(DECIMAL(20, 4))
    foreign_num_acc_yoy = Column(DECIMAL(10, 4))
    foreign_share_num_acc = Column(DECIMAL(20, 4))
    foreign_share_num_acc_yoy = Column(DECIMAL(10, 4))
    value_acc = Column(DECIMAL(20, 4))
    value_acc_yoy = Column(DECIMAL(10, 4))
    joint_value_acc = Column(DECIMAL(20, 4))
    joint_value_acc_yoy = Column(DECIMAL(10, 4))
    cooperative_value_acc = Column(DECIMAL(20, 4))
    cooperative_value_acc_yoy = Column(DECIMAL(10, 4))
    foreign_value_acc = Column(DECIMAL(20, 4))
    foreign_value_acc_yoy = Column(DECIMAL(10, 4))
    foreign_share_value_acc = Column(DECIMAL(20, 4))
    foreign_share_value_acc_yoy = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_MONEY_SUPPLY_YEAR(Base):
    """
    货币供应量(年度)
    """

    __tablename__ = "MAC_MONEY_SUPPLY_YEAR"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    m2 = Column(DECIMAL(20, 4))
    m1 = Column(DECIMAL(20, 4))
    m0 = Column(DECIMAL(20, 4))
    demond_deposit = Column(DECIMAL(20, 4))
    quosi = Column(DECIMAL(20, 4))
    time_deposit = Column(DECIMAL(20, 4))
    saving_deposit = Column(DECIMAL(20, 4))
    other_deposit = Column(DECIMAL(20, 4))
    m2_yoy = Column(DECIMAL(10, 4))
    m1_yoy = Column(DECIMAL(10, 4))
    m0_yoy = Column(DECIMAL(10, 4))
    demond_deposit_yoy = Column(DECIMAL(10, 4))
    quosi_yoy = Column(DECIMAL(10, 4))
    time_deposit_yoy = Column(DECIMAL(10, 4))
    saving_deposit_yoy = Column(DECIMAL(10, 4))
    other_deposit_yoy = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_STATS_REPORT_CALENDAR(Base):
    """
    国家统计局发布经济信息的日程表
    """

    __tablename__ = "MAC_STATS_REPORT_CALENDAR"

    id = Column(Integer, primary_key=True)
    report_date = Column(TIMESTAMP)
    content = Column(String(200), nullable=False)
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_INDUSTRY_AREA_ESTATE_BUILD_MONTH(Base):
    """
    各地区房地产开发规模与开、竣工面积增长情况表(月度累计)
    """
    __tablename__ = "MAC_INDUSTRY_AREA_ESTATE_BUILD_MONTH"

    id = Column(Integer, primary_key=True)
    stat_month = Column(String(20), nullable=False)
    area_code = Column(String(20), nullable=False)
    area_name = Column(String(100), nullable=False)
    construct_area = Column(DECIMAL(20, 4))
    construct_area_yoy = Column(DECIMAL(10, 4))
    new_start_area = Column(DECIMAL(20, 4))
    new_start_area_yoy = Column(DECIMAL(10, 4))
    complete_area = Column(DECIMAL(20, 4))
    complete_area_yoy = Column(DECIMAL(10, 4))
    resident_construct_area = Column(DECIMAL(20, 4))
    resident_construct_area_yoy = Column(DECIMAL(10, 4))
    resident_new_start_area = Column(DECIMAL(20, 4))
    resident_new_start_area_yoy = Column(DECIMAL(10, 4))
    resident_complete_area = Column(DECIMAL(20, 4))
    resident_complete_area_yoy = Column(DECIMAL(10, 4))
    office_construct_area = Column(DECIMAL(20, 4))
    office_construct_area_yoy = Column(DECIMAL(10, 4))
    office_new_start_area = Column(DECIMAL(20, 4))
    office_new_start_area_yoy = Column(DECIMAL(10, 4))
    office_complete_area = Column(DECIMAL(20, 4))
    office_complete_area_yoy = Column(DECIMAL(10, 4))
    business_construct_area = Column(DECIMAL(20, 4))
    business_construct_area_yoy = Column(DECIMAL(10, 4))
    business_new_start_area = Column(DECIMAL(20, 4))
    business_new_start_area_yoy = Column(DECIMAL(10, 4))
    business_complete_area = Column(DECIMAL(20, 4))
    business_complete_area_yoy = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addtime = Column(TIMESTAMP, default=datetime.datetime.now)
    modtime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_ENTERPRISE_BOOM_CONFIDENCE_IDX(Base):
    """
    企业景气及企业家信心指数
    """

    __tablename__ = "MAC_ENTERPRISE_BOOM_CONFIDENCE_IDX"

    id = Column(Integer, primary_key=True)
    stat_quarter = Column(String(20), nullable=False)
    boom_idx = Column(DECIMAL(10, 4))
    boom_idx_yoy = Column(DECIMAL(10, 4))
    boom_idx_mom = Column(DECIMAL(10, 4))
    confidence_idx = Column(DECIMAL(10, 4))
    confidence_idx_yoy = Column(DECIMAL(10, 4))
    confidence_idx_mom = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_AREA_GDP_YEAR_IDX_1978(Base):
    """
    分地区国内生产总值指数表(1978年=100)
    """

    __tablename__ = "MAC_AREA_GDP_YEAR_IDX_1978"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    area_code = Column(String(20), nullable=False)
    area_name = Column(String(100), nullable=False)
    gni_idx = Column(DECIMAL(10, 4))
    gdp_idx = Column(DECIMAL(10, 4))
    gdp_primary_idx = Column(DECIMAL(10, 4))
    gdp_agro_idx = Column(DECIMAL(10, 4))
    gdp_secondary_idx = Column(DECIMAL(10, 4))
    gdp_industry_idx = Column(DECIMAL(10, 4))
    gdp_construction_idx = Column(DECIMAL(10, 4))
    gdp_tertiary_idx = Column(DECIMAL(10, 4))
    gdp_sale_idx = Column(DECIMAL(10, 4))
    gdp_transport_idx = Column(DECIMAL(10, 4))
    gdp_hotel_idx = Column(DECIMAL(10, 4))
    gdp_financial_idx = Column(DECIMAL(10, 4))
    gdp_estate_idx = Column(DECIMAL(10, 4))
    gdp_others_idx = Column(DECIMAL(10, 4))
    gdp_per_capita_idx = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_AREA_GDP_EXPEND_YEAR(Base):
    """
    分地区支出法国内生产总值表(年度)
    """

    __tablename__ = "MAC_AREA_GDP_EXPEND_YEAR"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    area_code = Column(String(20), nullable=False)
    area_name = Column(String(100), nullable=False)
    gdp_expend = Column(DECIMAL(20, 4))
    gdp_fin_comsum_expend = Column(DECIMAL(20, 4))
    gdp_household_expend = Column(DECIMAL(20, 4))
    gdp_ruarl_household_expend = Column(DECIMAL(20, 4))
    gdp_citizen_household_expend = Column(DECIMAL(20, 4))
    gdp_gov_expend = Column(DECIMAL(20, 4))
    gdp_gross_capital_format = Column(DECIMAL(20, 4))
    gdp_gross_fixed_format = Column(DECIMAL(20, 4))
    gdp_gross_inventory_format = Column(DECIMAL(20, 4))
    gdp_net_export = Column(DECIMAL(20, 4))
    final_consum_rate = Column(DECIMAL(10, 4))
    capital_format_rate = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_FISCAL_EXTRA_REVENUE_EXPENSE_YEAR(Base):
    """
    预算外资金分项目收支表
    """
    __tablename__ = "MAC_FISCAL_EXTRA_REVENUE_EXPENSE_YEAR"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    revenue = Column(DECIMAL(20, 4))
    revenue_administration = Column(DECIMAL(20, 4))
    revenue_fund = Column(DECIMAL(20, 4))
    revenue_pool_fund = Column(DECIMAL(20, 4))
    revenue_local_finance = Column(DECIMAL(20, 4))
    revenue_state_enterprise = Column(DECIMAL(20, 4))
    revenue_other = Column(DECIMAL(20, 4))
    expense = Column(DECIMAL(20, 4))
    expense_pubic_service = Column(DECIMAL(20, 4))
    expense_education = Column(DECIMAL(20, 4))
    expense_security_employment = Column(DECIMAL(20, 4))
    expense_transportation = Column(DECIMAL(20, 4))
    expense_community_affairs = Column(DECIMAL(20, 4))
    expense_other = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addtime = Column(TIMESTAMP, default=datetime.datetime.now)
    modtime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_MANUFACTURING_PMI(Base):
    """
    制造业采购经理指数
    """

    __tablename__ = "MAC_MANUFACTURING_PMI"

    id = Column(Integer, primary_key=True)
    stat_month = Column(String(20), nullable=False)
    pmi = Column(DECIMAL(10, 4))
    produce_idx = Column(DECIMAL(10, 4))
    new_orders_idx = Column(DECIMAL(10, 4))
    new_export_orders_idx = Column(DECIMAL(10, 4))
    order_in_hand_idx = Column(DECIMAL(10, 4))
    finished_produce_idx = Column(DECIMAL(10, 4))
    purchase_quantity_idx = Column(DECIMAL(10, 4))
    import_idx = Column(DECIMAL(10, 4))
    exfactory_idx = Column(DECIMAL(10, 4))
    purchases_idx = Column(DECIMAL(10, 4))
    raw_material_idx = Column(DECIMAL(10, 4))
    employ_idx = Column(DECIMAL(10, 4))
    delivery_time_idx = Column(DECIMAL(10, 4))
    production_expected_idx = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_EMPLOY_YEAR(Base):
    """
    就业情况基本表(年度)
    """

    __tablename__ = "MAC_EMPLOY_YEAR"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    population = Column(DECIMAL(20, 4))
    employ = Column(DECIMAL(20, 4))
    employ_primary = Column(DECIMAL(20, 4))
    employ_secondary = Column(DECIMAL(20, 4))
    employ_tertiary = Column(DECIMAL(20, 4))
    employ_urban = Column(DECIMAL(20, 4))
    employ_urban_state_owned = Column(DECIMAL(20, 4))
    employ_urban_collective = Column(DECIMAL(20, 4))
    employ_urban_stock_cooperate = Column(DECIMAL(20, 4))
    employ_urban_joint_ownership = Column(DECIMAL(20, 4))
    employ_urban_limited = Column(DECIMAL(20, 4))
    employ_urban_stock = Column(DECIMAL(20, 4))
    employ_urban_private = Column(DECIMAL(20, 4))
    employ_urban_hkmt = Column(DECIMAL(20, 4))
    employ_urban_foreign = Column(DECIMAL(20, 4))
    employ_urban_individual = Column(DECIMAL(20, 4))
    employ_rural = Column(DECIMAL(20, 4))
    employ_rural_private = Column(DECIMAL(20, 4))
    employ_rural_individual = Column(DECIMAL(20, 4))
    unemploy_num = Column(DECIMAL(20, 4))
    unemploy_rate = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addTime = Column(TIMESTAMP, default=datetime.datetime.now)
    modTime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_PARAMETER(Base):
    """
    宏观经济数据-编码表
    """
    __tablename__ = "MAC_PARAMETER"

    id = Column(Integer, primary_key=True)
    class_code = Column(Integer, nullable=False)
    class_name = Column(String(100), nullable=False)
    parameter_code = Column(Integer, nullable=False)
    parameter_name = Column(String(100), nullable=False)
    description = Column(String(200))
    status = Column(TINYINT(display_width=4), default=0)
    addtime = Column(TIMESTAMP, default=datetime.datetime.now)
    modtime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_POPULATION_DEPENDENCY(Base):
    """
    人口年龄结构和抚养比例表
    """
    __tablename__ = "MAC_POPULATION_DEPENDENCY"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    population = Column(DECIMAL(20, 4))
    age_between_0_and_14 = Column(DECIMAL(20, 4))
    age_between_15_and_64 = Column(DECIMAL(20, 4))
    age_over_65 = Column(DECIMAL(20, 4))
    dependency_ratio = Column(DECIMAL(10, 4))
    children_dependency_ratio = Column(DECIMAL(10, 4))
    old_dependency_ratio = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addtime = Column(TIMESTAMP, default=datetime.datetime.now)
    modtime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_INDUSTRY_ESTATE_70CITY_INDEX_MONTH(Base):
    """
    70个大中城市房屋销售价格指数(月度)
    """
    __tablename__ = "MAC_INDUSTRY_ESTATE_70CITY_INDEX_MONTH"

    id = Column(Integer, primary_key=True)
    stat_month = Column(String(20), nullable=False)
    area_code = Column(String(20), nullable=False)
    area_name = Column(String(100), nullable=False)
    fixed_base_type = Column(String(100), nullable=False)
    resident_idx = Column(DECIMAL(10, 4))
    commodity_house_idx = Column(DECIMAL(10, 4))
    second_hand_idx = Column(DECIMAL(10, 4))
    commodity_house_below90_idx = Column(DECIMAL(10, 4))
    second_hand_below90_idx = Column(DECIMAL(10, 4))
    commodity_house_between_90_140_idx = Column(DECIMAL(10, 4))
    second_hand_between_90_140_idx = Column(DECIMAL(10, 4))
    commodity_house_above140_idx = Column(DECIMAL(10, 4))
    second_house_above140_idx = Column(DECIMAL(10, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addtime = Column(TIMESTAMP, default=datetime.datetime.now)
    modtime = Column(TIMESTAMP)


# coding: utf-8
import datetime
from sqlalchemy import BigInteger, Column, DateTime, Integer, Numeric, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql import TINYINT, TIMESTAMP, DECIMAL
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class MAC_FIXED_INVESTMENT_YEAR(Base):
    """
    固定资产投资情况表(年度)
    """
    __tablename__ = "MAC_FIXED_INVESTMENT_YEAR"

    id = Column(Integer, primary_key=True)
    stat_year = Column(String(4), nullable=False)
    fixed_investment = Column(DECIMAL(20, 4))
    urban_fixed_investment = Column(DECIMAL(20, 4))
    mainland = Column(DECIMAL(20, 4))
    state_owned = Column(DECIMAL(20, 4))
    collective = Column(DECIMAL(20, 4))
    joint_stock = Column(DECIMAL(20, 4))
    joint_owned = Column(DECIMAL(20, 4))
    limited = Column(DECIMAL(20, 4))
    stock = Column(DECIMAL(20, 4))
    private = Column(DECIMAL(20, 4))
    individual = Column(DECIMAL(20, 4))
    others = Column(DECIMAL(20, 4))
    hkmt = Column(DECIMAL(20, 4))
    foreign = Column(DECIMAL(20, 4))
    state_budget = Column(DECIMAL(20, 4))
    domestic_loan = Column(DECIMAL(20, 4))
    foreign_investment = Column(DECIMAL(20, 4))
    self_raised_fund = Column(DECIMAL(20, 4))
    other_fund = Column(DECIMAL(20, 4))
    construct_install = Column(DECIMAL(20, 4))
    equipment_purchase = Column(DECIMAL(20, 4))
    other_expense = Column(DECIMAL(20, 4))
    status = Column(TINYINT(display_width=4), default=0)
    addtime = Column(TIMESTAMP, default=datetime.datetime.now)
    modtime = Column(TIMESTAMP)